=== modified file 'merge.py'
--- merge.py	2008-07-25 15:42:45 +0000
+++ merge.py	2008-07-27 11:20:34 +0000
@@ -45,44 +45,83 @@
         # Get arguments
         self.wt = wt
         self.wtpath = wtpath
-        
-        if default_branch_path and os.path.isdir(default_branch_path.partition('file://')[2]):
-            directory = default_branch_path.partition('file://')[2]
-        else:
-            directory = os.path.dirname(self.wt.abspath(self.wtpath))
+        self.default_branch_path = default_branch_path
+        self.parent_window = parent
         
         # Create widgets
         self._hbox = gtk.HBox()
+        self._source = gtk.HBox()
         self._label_merge_from = gtk.Label(_i18n("Merge from"))
-        self._filechooser_dialog = gtk.FileChooserDialog(title="Please select a folder",
-                                    parent=self.window,
-                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
-        self._filechooser_dialog.set_default_response(gtk.RESPONSE_OK)
-        self._filechooser = gtk.FileChooserButton(self._filechooser_dialog)
+        self._combo_source = gtk.combo_box_new_text()
+        for entry in [_i18n("Folder"),_i18n("Custom Location")]:
+            self._combo_source.append_text(entry)
+        self._combo_source.connect("changed", self._on_combo_changed)
         self._button_merge = gtk.Button(_i18n("_Merge"))
         self._button_merge_icon = gtk.Image()
         self._button_merge_icon.set_from_stock(gtk.STOCK_APPLY, gtk.ICON_SIZE_BUTTON)
         self._button_merge.set_image(self._button_merge_icon)
-        
         self._button_merge.connect('clicked', self._on_merge_clicked)
         
-        # Set location
-        self._filechooser_dialog.set_current_folder(directory)
-        
         # Add widgets to dialog
-        self.vbox.add(self._hbox)
+        self.vbox.pack_start(self._hbox, False, False, 0)
         self._hbox.add(self._label_merge_from)
-        self._hbox.add(self._filechooser)
+        self._hbox.add(self._combo_source)
         self._hbox.set_spacing(5)
         self.action_area.pack_end(self._button_merge)
         
+        if self.default_branch_path and os.path.isdir(self.default_branch_path.partition('file://')[2]):
+            self.directory = self.default_branch_path.partition('file://')[2]
+            self._combo_source.set_active(0)
+        else:
+            self._on_custom_source()
         self.vbox.show_all()
-
+    
+    def _on_folder_source(self):
+        """ Merge from folder, create a filechooser dialog and button """
+        self._source = gtk.HBox()
+        self._filechooser_dialog = gtk.FileChooserDialog(title="Please select a folder",
+                                    parent=self.parent_window,
+                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                                             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        self._filechooser_dialog.set_default_response(gtk.RESPONSE_OK)
+        self._filechooser = gtk.FileChooserButton(self._filechooser_dialog)
+        self._filechooser.show()
+        directory = getattr(self, 'directory', None)
+        if not directory:
+            directory = os.path.dirname(self.wt.abspath(self.wtpath))
+        self._filechooser_dialog.set_current_folder(directory)
+        self._source.pack_start(self._filechooser, True, True, 0)
+        self.vbox.pack_start(self._source, True, True, 5)
+        self._source.show()
+    
+    def _on_custom_source(self):
+        """ Merge from a custom source (can be folder, remote, etc), create entry """
+        self._source = gtk.HBox()
+        self._custom_entry = gtk.Entry()
+        self._custom_entry.set_text(self.default_branch_path)
+        self._custom_entry.show()
+        self._source.pack_start(self._custom_entry, True, True, 0)
+        self.vbox.pack_start(self._source, True, True, 5)
+        self._source.show()
+    
+    def _on_combo_changed(self, widget):
+        merge_source = self._combo_source.get_active()
+        self._source.destroy()
+        if merge_source == 0:
+            # Merge from folder
+            self._on_folder_source()
+        elif merge_source == 1:
+            # Merge from custom
+            self._on_custom_source()
+    
     @show_bzr_error
     def _on_merge_clicked(self, widget):
-        branch = self._filechooser.get_filename()
+        merge_source = self._combo_source.get_active()
+        if merge_source == 0:
+            branch = self._filechooser.get_filename()
+        elif merge_source == 1:
+            branch = self._custom_entry.get_text()
         if branch == "":
             error_dialog(_i18n('Branch not given'),
                          _i18n('Please specify a branch to merge from.'))

